@@ -43,8 +43,13 @@ struct VTable
             void* data)
     {
         auto thiz = static_cast<VTable*>(data);
+        auto sp = thiz->object.lock();
 
-        if (thiz->object->on_new_message(
+        // We return early if the underlying object is dead.
+        if (not sp)
+            return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+        if (sp->on_new_message(
                     core::dbus::Message::from_raw_message(
                         message)))
             return DBUS_HANDLER_RESULT_HANDLED;
@@ -52,7 +57,7 @@ struct VTable
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
 
-    std::shared_ptr<core::dbus::Object> object;
+    std::weak_ptr<core::dbus::Object> object;
 };
 
 DBusHandlerResult static_handle_message(
@@ -273,7 +278,9 @@ std::shared_ptr<Message> Bus::send_with_reply_and_block_for_at_most(
     if (!result)
         throw std::runtime_error(se.print());
 
-    return Message::from_raw_message(result);
+    auto reply = Message::from_raw_message(result);
+    dbus_message_unref(result);
+    return reply;
 }
 
 PendingCall::Ptr Bus::send_with_reply_and_timeout(
